package de.arvato.systems.nmvs.example.v3;

import de.arvato.systems.nmvs.example.Main;
import eu.nmvs.types.v3.BaseBatchType;
import eu.nmvs.types.v3.CatalogProductSchemeType;
import eu.nmvs.types.v3.CatalogUseCaseMixedType;
import eu.nmvs.types.v3.ChangePwdRequestDataType;
import eu.nmvs.types.v3.DownloadCertRequestDataType;
import eu.nmvs.types.v3.I6ResultTransactionDataType;
import eu.nmvs.types.v3.I9DownloadMasterdataTransactionDataType;
import eu.nmvs.types.v3.I9DownloadMasterdataTransactionDataType.ProductCodeList;
import eu.nmvs.types.v3.ManualRequestDataType;
import eu.nmvs.types.v3.ManualRequestDataType.Product;
import eu.nmvs.types.v3.MixedBulkRequestTransactionType;
import eu.nmvs.types.v3.MixedSingleType;
import eu.nmvs.types.v3.ProductIdentifierType;
import eu.nmvs.types.v3.RequestAuthHeaderDataType;
import eu.nmvs.types.v3.RequestBulkDataType;
import eu.nmvs.types.v3.RequestBulkDataType.Packs;
import eu.nmvs.types.v3.RequestDataType;
import eu.nmvs.types.v3.RequestHeaderDataType;
import eu.nmvs.types.v3.RequestMixedBulkDataType;
import eu.nmvs.types.v3.RequestMixedBulkDataType.TrxList;
import eu.nmvs.types.v3.RequestMixedBulkDataType.TrxList.TrxItem;
import eu.nmvs.types.v3.RequestPackType;
import eu.nmvs.types.v3.RequestProductType;
import eu.nmvs.types.v3.RequestTransactionHeaderDataType;
import eu.nmvs.types.v3.RequestUndoBulkDataType;
import eu.nmvs.types.v3.RequestUndoManualSingleDataType;
import eu.nmvs.types.v3.RequestUndoSingleDataType;
import eu.nmvs.types.v3.UserSoftwareType;

/**
 * This is a utility class to create request objects and fill them with test data.<br>
 * The example classes will use these methods to setup their web service calls.
 * <p>
 * In <em>your</em> code, you would use your own business objects to fill these.
 */
final class SOAPObjectCreator {

    private static final String CLIENT_TRANSACTION_ID = "06c8dea069c24446949e7806e028bcaf";

    RequestAuthHeaderDataType createAuth() {
        RequestAuthHeaderDataType auth = new RequestAuthHeaderDataType();
        auth.setClientLoginId(Main.WS_PROPERTIES.getProperty("clientLoginId"));
        auth.setPassword(Main.WS_PROPERTIES.getProperty("password"));
        auth.setSubUserId(Main.WS_PROPERTIES.getProperty("subUserId"));
        auth.setUserId(Main.WS_PROPERTIES.getProperty("userId"));
        return auth;
    }

    RequestDataType createBody() {
        RequestDataType body = new RequestDataType();
        body.setPack(createPack());
        body.setProduct(createProduct());
        return body;
    }

    DownloadCertRequestDataType createBodyCertRequest() {
        DownloadCertRequestDataType body = new DownloadCertRequestDataType();
        body.setTan(CLIENT_TRANSACTION_ID);
        return body;
    }

    ChangePwdRequestDataType createBodyChangePwd() {
        ChangePwdRequestDataType body = new ChangePwdRequestDataType();
        body.setNewPassword("87654321");
        body.setPassword("12345678");
        return body;
    }

    RequestBulkDataType createBodyBulk() {
        RequestBulkDataType body = new RequestBulkDataType();
        body.setPacks(createPacksBulk());
        body.setProduct(createProduct());
        return body;
    }

    I6ResultTransactionDataType createBodyI6() {
        I6ResultTransactionDataType body = new I6ResultTransactionDataType();
        // Changed from v1
        body.setRefNMVSTrxId(CLIENT_TRANSACTION_ID);
        return body;
    }

    I9DownloadMasterdataTransactionDataType createBodyI9() {
        ProductCodeList productCodeList = new ProductCodeList();
        productCodeList.getProductCode().add(createProductId());

        I9DownloadMasterdataTransactionDataType body = new I9DownloadMasterdataTransactionDataType();
        body.setProductCodeList(productCodeList);
        return body;
    }

    ManualRequestDataType createBodyManual() {
        Product prod = new Product();
        prod.setProductCode(createProductId());

        ManualRequestDataType body = new ManualRequestDataType();
        body.setPack(createPack());
        body.setProduct(prod);
        return body;
    }

    RequestMixedBulkDataType createBodyMixed() {
        MixedBulkRequestTransactionType tx = new MixedBulkRequestTransactionType();
        tx.setClientTrxId(CLIENT_TRANSACTION_ID);
        tx.setLanguage("deu");
        tx.setRefClientTrxId(CLIENT_TRANSACTION_ID);

        MixedSingleType item = new MixedSingleType();
        item.setPack(createPack());
        item.setProduct(createProduct());
        item.setReqType(CatalogUseCaseMixedType.G_110);
        item.setTransaction(tx);

        TrxItem trxItem = new TrxItem();
        trxItem.setItem(item);

        TrxList trxList = new TrxList();
        trxList.getTrxItem().add(trxItem);

        RequestMixedBulkDataType body = new RequestMixedBulkDataType();
        body.setTrxList(trxList);
        return body;
    }

    RequestUndoSingleDataType createBodyUndo() {
        RequestUndoSingleDataType body = new RequestUndoSingleDataType();
        body.setPack(createPack());
        body.setProduct(createProduct());
        body.setRefClientTrxId(CLIENT_TRANSACTION_ID);
        return body;
    }

    RequestUndoBulkDataType createBodyUndoBulk() {
        RequestUndoBulkDataType body = new RequestUndoBulkDataType();
        body.setPacks(createPacksBulk());
        body.setProduct(createProduct());
        body.setRefClientTrxId(CLIENT_TRANSACTION_ID);
        return body;
    }

    RequestHeaderDataType createHeader() {
        RequestHeaderDataType header = new RequestHeaderDataType();
        header.setAuth(createAuth());
        header.setTransaction(createTransaction());
        header.setUserSoftware(createUserSoftware());
        return header;
    }

    RequestPackType createPack() {
        RequestPackType pack = new RequestPackType();
        pack.setSn("AZAZAZA");
        return pack;
    }

    Packs createPacksBulk() {
        RequestPackType pack = new RequestPackType();
        pack.setSn("123456");

        Packs packs = new Packs();
        packs.getPack().add(pack);
        return packs;
    }

    RequestProductType createProduct() {
        BaseBatchType batch = new BaseBatchType();
        batch.setExpDate("170721"); // YYMMDD
        batch.setId("1");

        RequestProductType product = new RequestProductType();
        product.setBatch(batch);
        product.setProductCode(createProductId());
        return product;
    }

    ProductIdentifierType createProductId() {
        ProductIdentifierType prodId = new ProductIdentifierType();
        prodId.setScheme(CatalogProductSchemeType.GTIN);
        prodId.setValue("1234567891234");
        return prodId;
    }

    RequestTransactionHeaderDataType createTransaction() {
        RequestTransactionHeaderDataType transaction = new RequestTransactionHeaderDataType();
        transaction.setClientTrxId(CLIENT_TRANSACTION_ID);
        transaction.setLanguage("deu");
        return transaction;
    }

    UserSoftwareType createUserSoftware() {
        UserSoftwareType userSoftware = new UserSoftwareType();
        userSoftware.setName("EasyPharm");
        userSoftware.setSupplier("PharmSoft Ltd.");
        userSoftware.setVersion("3.2.0");
        return userSoftware;
    }

    protected RequestUndoManualSingleDataType createBodyUndoManual() {
        RequestUndoManualSingleDataType undoManualSingle = new RequestUndoManualSingleDataType();
        Product product = new Product();
        product.setProductCode(createProductId());
        undoManualSingle.setProduct(product);
        undoManualSingle.setRefClientTrxId(CLIENT_TRANSACTION_ID);
        undoManualSingle.setPack(createPack());
        return undoManualSingle;
    }

}
