package de.arvato.systems.nmvs.example.v3;

import java.math.BigInteger;

import de.arvato.systems.nmvs.example.AbstractExample;
import eu.nmvs.services.v3_0.ISupportServices;
import eu.nmvs.services.v3_0.SupportServices;
import eu.nmvs.types.v3.ConfirmTermsAndConditionsRequestDataType;
import eu.nmvs.wsdltypes.v3.G445Request;
import eu.nmvs.wsdltypes.v3.I12ConfirmTermsAndConditionsType;
import eu.nmvs.wsdltypes.v3.I7RequestType;
import eu.nmvs.wsdltypes.v3.SupportPingRequest;

/**
 * This is an example class for the web service described by the <em>WS_SUPPORT.wsdl</em>.
 * It provides a method for each web method. {@link AbstractExample} provides the {@link AbstractExample#invoke()} method
 * wich prompts the user for a method to call and then calls it reflectively.
 *  
 * @author ZAND026
 */
public class WsSupportExample extends AbstractExampleV3<ISupportServices> {

    @Override
    protected Class<ISupportServices> getWSInterfaceClass() {
        return ISupportServices.class;
    }

    @Override
    public String getWsName() {
        return SupportServices.class.getSimpleName();
    }

    public void g445ChangePassword() {
        G445Request req = new G445Request();
        req.setBody(soapObjectCreator.createBodyChangePwd());
        req.setHeader(createHeader());
        getPort().g445ChangePassword(req);
    }

    public void g482LoadTermsAndConditions() {
        I7RequestType messageParameters = createI7RequestType();
        getPort().g482LoadTermsAndConditions(messageParameters);
    }

    public void g483ConfirmTermsAndConditions() {
        I12ConfirmTermsAndConditionsType messageParameters = new I12ConfirmTermsAndConditionsType();
        ConfirmTermsAndConditionsRequestDataType confirmTermsAndConditionsRequestDataType = new ConfirmTermsAndConditionsRequestDataType();
        confirmTermsAndConditionsRequestDataType.setVersion(BigInteger.valueOf(42L));
        messageParameters.setBody(confirmTermsAndConditionsRequestDataType);
        messageParameters.setHeader(createHeader());
        getPort().g483ConfirmTermsAndConditions(messageParameters);
    }

    public void g487LoadDataPrivacyPolicies() {
        getPort().g487LoadDataPrivacyPolicies(createI7RequestType());
    }

    public void pingSupport() {
        SupportPingRequest req = new SupportPingRequest();
        req.setInput("Hello World!");
        getPort().pingSupport(req);
    }

    protected I7RequestType createI7RequestType() {
        I7RequestType messageParameters = new I7RequestType();
        messageParameters.setHeader(createHeader());
        return messageParameters;
    }

}
