﻿using System;
using NMVS_Sample.BulkServices;
using System.ServiceModel;
using System.Security.Cryptography.X509Certificates;
using System.Collections.Generic;

namespace NMVS_Sample.Bulk
{
    class G127
    {
        public static object Header { get; set; }

        /// <summary>
        /// Creates a request to undo an Bulk dispense
        /// </summary>
        public static void G127BulkUndoDispense()
        {
            //add using directive NMVS_Sample.BulkServices;
            //binding the service address you want to contact
            string EndPoint = Properties.Settings.Default.BulkServicesEndPoint;

            //Creates a G127Request object and a G127BulkUndoDispenseRequest object
            G127BulkUndoDispenseRequest request = new G127BulkUndoDispenseRequest();
            G127Request g127Request = new G127Request();

            //Executes the SetHeaderRequest method from the header class to fill the header
            object Header = NMVS_Sample.Header.SetHeaderRequest("NMVS_Sample.BulkServices", false);
            //Executes the SetBulkDispenseUndoRequestBody method to fill the body
            object Body = SetBulkDispenseUndoRequestBody();

            //Appends the header and body to the request object
            g127Request.Header = (RequestHeaderData_Type)Header;
            g127Request.Body = (RequestUndoBulkData_Type)Body;
            request.G127Request = g127Request;

            //add using directive System.ServiceModel;
            //Defines a secure binding with certificate authentication
            WSHttpBinding binding = new WSHttpBinding();
            binding.Security.Mode = SecurityMode.Transport;
            binding.Security.Transport.ClientCredentialType = HttpClientCredentialType.Certificate;

            //Creates ServiceClient, attach transport-binding, Endpoint and the loaded certificate
            BulkServicesClient service = new BulkServicesClient(binding, new EndpointAddress(EndPoint));

            //add using directive System.Security.Cryptography.X509Certificates;
            X509Certificate2 cert = new X509Certificate2(Properties.Settings.Default.CertificateFilePath, Properties.Settings.Default.PrivateKeyPassword, X509KeyStorageFlags.PersistKeySet);
            service.ClientCredentials.ClientCertificate.Certificate = cert;
            service.Endpoint.EndpointBehaviors.Add(new CustomMessageInspector());

            //Creates G127Response, result of the request.
            G127Response response = service.G127BulkUndoDispense(g127Request);

            //Displays the response header and body.
            Console.WriteLine(response.Header);
            Console.WriteLine(response.Body);
        }

        /// <summary>
        /// Method to generate the body to undo an Bulk dispense
        /// </summary>
        /// <returns></returns>
        private static object SetBulkDispenseUndoRequestBody()
        {
            BulkServices.RequestPack_Type pack = new BulkServices.RequestPack_Type();
            List<RequestPack_Type> mylist = new List<RequestPack_Type>();
            BulkServices.RequestPack_Type[] Packs;
            pack.sn = "ABCDEFG";//A simple way to create multiple serial numbers to simulate the functionality of BulkRequest
            mylist.Add(pack);
            pack = new BulkServices.RequestPack_Type();
            pack.sn = "HIJKLMN";
            mylist.Add(pack);
            //pack = new BulkServices.RequestPack_Type();
            //pack.sn = "OPQRSTU";
            //mylist.Add(pack);
            //...
            Packs = mylist.ToArray();

            //Set the product code and the schema of a single pack
            BulkServices.ProductIdentifier_Type productCode = new BulkServices.ProductIdentifier_Type();
            productCode.Value = "1234567891234";
            productCode.scheme = BulkServices.CatalogProductScheme_Type.GTIN; // Alternativ PPN

            //Set batch id and expiration date
            BulkServices.BaseBatch_Type batch = new BulkServices.BaseBatch_Type();
            batch.Id = "1";
            batch.ExpDate = "170721";

            //Append the productcode and batch objects to the RequestProduct_Type object
            BulkServices.RequestProduct_Type product = new BulkServices.RequestProduct_Type();
            product.ProductCode = productCode;
            product.Batch = batch;

            //Append the pack and product object to the RequestUndoBulkData_Type object, and set the guid as RefClientTrxId
            BulkServices.RequestUndoBulkData_Type bodyUndo = new BulkServices.RequestUndoBulkData_Type();
            bodyUndo.Packs = Packs;
            bodyUndo.Product = product;
            //Optional: bodyUndo.RefClientTrxId = Guid.NewGuid().ToString().Replace("-", "");
            return bodyUndo;
        }
    }
}
