﻿using System;
using NMVS_Sample.SinglePackServices;
using System.ServiceModel;
using System.Security.Cryptography.X509Certificates;


namespace NMVS_Sample.SinglePack
{
    public class G140
    {
        public static object Header { get; set; }
        /// <summary>
        /// Creates a singlepack export request
        /// </summary>
        public static void G140Export()
        {
            //add using directive NMVS_Sample.SinglePackServices;
            //binding the service address you want to contact
            string EndPoint = Properties.Settings.Default.SinglePackServicesEndPoint;

            //Creates a G140Request object and a G140ExportRequest object
            G140ExportRequest request = new G140ExportRequest();
            G140Request g140Request = new G140Request();

            //Executes the SetHeaderRequest method from the header class to fill the header
            object Header = NMVS_Sample.Header.SetHeaderRequest("NMVS_Sample.SinglePackServices", false);
            //Executes the SetSinglePackExportRequestBody method to fill the body
            object Body = SetSinglePackExportRequestBody();

            //Appends the header and body to the request object
            g140Request.Header = (RequestHeaderData_Type)Header;
            g140Request.Body = (RequestData_Type)Body;
            request.G140Request = g140Request;

            //add using directive System.ServiceModel;
            //Defines a secure binding with certificate authentication
            WSHttpBinding binding = new WSHttpBinding();
            binding.Security.Mode = SecurityMode.Transport;
            binding.Security.Transport.ClientCredentialType = HttpClientCredentialType.Certificate;

            //Creates ServiceClient, attach transport-binding, Endpoint and the loaded certificate
            SinglePackServicesClient service = new SinglePackServicesClient(binding, new EndpointAddress(EndPoint));

            //add using directive System.Security.Cryptography.X509Certificates;
            X509Certificate2 cert = new X509Certificate2(Properties.Settings.Default.CertificateFilePath, Properties.Settings.Default.PrivateKeyPassword, X509KeyStorageFlags.PersistKeySet);
            service.ClientCredentials.ClientCertificate.Certificate = cert;
            service.Endpoint.EndpointBehaviors.Add(new CustomMessageInspector());

            //Creates G140Response, result of the request.
            G140Response response = service.G140Export(g140Request);

            //Displays the response header and body.
            Console.WriteLine(response.Header);
            Console.WriteLine(response.Body);
        }

        /// <summary>
        /// Method to generate the body for a Singlepack export request
        /// </summary>
        /// <returns></returns>
        private static object SetSinglePackExportRequestBody()
        {
            //Set the serial number of a single pack
            SinglePackServices.RequestPack_Type pack = new SinglePackServices.RequestPack_Type();
            pack.sn = "ABCDEFG";

            //Set the product code and the schema of a single pack
            SinglePackServices.ProductIdentifier_Type productCode = new SinglePackServices.ProductIdentifier_Type();
            productCode.Value = "1234567891234";
            productCode.scheme = SinglePackServices.CatalogProductScheme_Type.GTIN; // Alternativ PPN

            //Set batch id and expiration date
            SinglePackServices.BaseBatch_Type batch = new SinglePackServices.BaseBatch_Type();
            batch.Id = "1";
            batch.ExpDate = "17";

            //Append the productcode and batch objects to the RequestProduct_Type object
            SinglePackServices.RequestProduct_Type product = new SinglePackServices.RequestProduct_Type();
            product.ProductCode = productCode;
            product.Batch = batch;

            //Append the pack and product object to the RequestData_Type object
            SinglePackServices.RequestData_Type body = new SinglePackServices.RequestData_Type();
            body.Pack = pack;
            body.Product = product;
            return body;
        }
    }

}
