﻿using System;
using NMVS_Sample.SinglePackServices;
using System.ServiceModel;
using System.Security.Cryptography.X509Certificates;


namespace NMVS_Sample.SinglePack
{
    public class G182
    {
        public static object Header { get; set; }
        /// <summary>
        /// Creates a stolen singlepack manual entry request
        /// </summary>
        public static void G182StolenManualEntry()
        {
            //add using directive NMVS_Sample.SinglePackServices;
            //binding the service address you want to contact
            string EndPoint = Properties.Settings.Default.SinglePackServicesEndPoint;

            //Creates a G182Request object and a G182StolenManualEntryRequest object
            G182StolenManualEntryRequest request = new G182StolenManualEntryRequest();
            G182Request g182Request = new G182Request();

            //Executes the SetHeaderRequest method from the header class to fill the header
            object Header = NMVS_Sample.Header.SetHeaderRequest("NMVS_Sample.SinglePackServices", false);
            //Executes the SetSinglePackStolenManualEntryRequestBody method to fill the body
            object Body = SetSinglePackStolenManualEntryRequestBody();

            //Appends the header and body to the request object
            g182Request.Header = (RequestHeaderData_Type)Header;
            g182Request.Body = (ManualRequestData_Type)Body;
            request.G182Request = g182Request;

            //add using directive System.ServiceModel;
            //Defines a secure binding with certificate authentication
            WSHttpBinding binding = new WSHttpBinding();
            binding.Security.Mode = SecurityMode.Transport;
            binding.Security.Transport.ClientCredentialType = HttpClientCredentialType.Certificate;

            //Creates ServiceClient, attach transport-binding, Endpoint and the loaded certificate
            SinglePackServicesClient service = new SinglePackServicesClient(binding, new EndpointAddress(EndPoint));

            //add using directive System.Security.Cryptography.X509Certificates;
            X509Certificate2 cert = new X509Certificate2(Properties.Settings.Default.CertificateFilePath, Properties.Settings.Default.PrivateKeyPassword, X509KeyStorageFlags.PersistKeySet);
            service.ClientCredentials.ClientCertificate.Certificate = cert;
            service.Endpoint.EndpointBehaviors.Add(new CustomMessageInspector());

            //Creates G182Response, result of the request.
            G182Response response = service.G182StolenManualEntry(g182Request);

            //Displays the response header and body.
            Console.WriteLine(response.Header);
            Console.WriteLine(response.Body);
        }

        /// <summary>
        /// Method to generate the body for a stolen Singlepack manual entry request
        /// </summary>
        /// <returns></returns>
        private static object SetSinglePackStolenManualEntryRequestBody()
        {
            //Set the serial number of a single pack
            SinglePackServices.RequestPack_Type pack = new SinglePackServices.RequestPack_Type();
            pack.sn = "ABCDEFG";

            //Set the product code and the schema of a single pack
            SinglePackServices.ProductIdentifier_Type productCode = new SinglePackServices.ProductIdentifier_Type();
            productCode.Value = "1234567891234";
            productCode.scheme = SinglePackServices.CatalogProductScheme_Type.GTIN; // Alternativ PPN

            //Append the productcode and batch objects to the RequestProduct_Type object
            SinglePackServices.ManualRequestData_TypeProduct product = new SinglePackServices.ManualRequestData_TypeProduct();
            product.ProductCode = productCode;

            //Append the pack and product object to the ManualRequestData_Type object
            ManualRequestData_Type body = new ManualRequestData_Type();
            body.Pack = pack;
            body.Product = product;
            return body;
        }
    }

}
